<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\Middleware;

use App\Repositories\Enums\ResponseCodeEnum;
use App\Services\RoleAndPermissionsService;
use Closure;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Contracts\Auth\Factory as Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;
use App\Support\Traits\Helpers;


class Authenticate
{
    use Helpers;
    /**
     * The authentication guard factory instance.
     *
     * @var Auth
     */
    protected $auth;

    protected $whiteList = [
        '/adminapi/user/info',
        '/adminapi/auth/logout',
        '/adminapi/permission/dictData/defaultButtonPermission',
        '/adminapi/permission/menu/userRoleMenuList'
    ];

    /**
     * Create a new middleware instance.
     *
     * @param  Auth  $auth
     */
    public function __construct(Auth $auth)
    {
        $this->auth = $auth;
    }

    /**
     * Handle an incoming request.
     *
     * @param  Request  $request
     * @param  Closure  $next
     * @param  string|null  $guard
     * @return mixed
     *
     * @throws AuthorizationException
     */
    public function handle($request, Closure $next, $guard = null)
    {
        /**
         * 如果路由是登出或者是修改密码的时候，需要对token进行处理，否则jwt处理时会出错
         */ 
        if(
            Str::endsWith(strtolower((string) $request->getPathInfo()), 'logout') ||
            Str::endsWith(strtolower((string) $request->getPathInfo()), 'resetpassword')
            )
        {

            $token = trim(str_ireplace('bearer', '', $request->header('authorization')));
            $decodeToken = $this->decodeToken($token);
            if(empty($decodeToken) || (!is_array($decodeToken) && !$decodeToken['hash'])){
                throw new UnauthorizedHttpException('JWTAuth', 'Unable to authenticate with invalid token.');
            }

            /*$chains = app('tymon.jwt.parser')->getChain();
            $chains[0]->setHeaderName($decodeToken['hash']);
            app('tymon.jwt.parser')->setChain($chains);*/

            $request->headers->set('authorization','bearer'.$decodeToken['hash']);
        }
        else{

            //对用户权限进行判断验证
            $currentUser = auth()->user();
            if(!$currentUser->isSuperAdmin()){
               if(!$this->userHasPermission($currentUser, $request)){
                    throw new \LogicException('你无权访问或使用该功能',ResponseCodeEnum::SYSTEM_ERROR);
                }
                return $next($request);
            }
        }

        return $next($request);
    }

    /**
     * 判断用户是否有访问当前路由的权限
     * @param $user
     * @param $request
     * @return bool
     */
    protected function userHasPermission($user, $request)
    {
        $userPermissions = [];

        $routeParams = $request->route()[1];

        if(!isset($routeParams['permission'])){
            return true;
        }
        $routePermissions = explode(',',$routeParams['permission']);
        // 如果该路由不存在权限，或者在白名单中，直接不做权限验证
        if(empty($routePermissions) || count($routePermissions) == 0){
            return true;
        }

        // 获取用户的所有角色组对应的权限
        $roles = $user->roles;
        collect($roles)->map(function($role) use (&$userPermissions){
            $permissions = $role->permissions;
            collect($permissions)->map(function($permission) use (&$userPermissions){
                $userPermissions[] = $permission->toArray()['action'];
            });
        });

        $userPermissions = array_values(array_filter($userPermissions));

        $hasPermission = false;
        // 获取 dingo 对应的版本的路由列表
        $routeList = [];
        collect(app('Dingo\Api\Contract\Routing\Adapter')->getIterableRoutes())->map(function($routes,$version) use ($request, $userPermissions, $routeParams, &$hasPermission){
            if($version != config('api.version')){
                return ;
            }

            // 此逻辑是，可能页面搜索和列表使用的是同一个request请求，如果用搜索来做请求的话，必须把权限设置为搜索权限，否则权限将出现漏洞
            // 支持泛权限 如果最后是*号结束，代表只要是前缀相同的都有访问资格
            if(
                !Str::endsWith(strtolower((string) $routeParams['permission']), '*'))
            {
                if(
                    $request->has('buttonAction') &&
                    $action = $request->input('buttonAction'))
                {
                    $uriPermission = explode(',',$routeParams['permission']);
                    $permission = explode('.',current($uriPermission));
                    $permission[count($permission) - 1] = $action;

                    $requestPermission = implode('.',$permission);

                    foreach($userPermissions as &$permission){
                        if(in_array($permission,$uriPermission)){
                            $permission = $requestPermission;
                        }
                    }
                }
            }

            $filterRoute = [];
            foreach($routes as $route){
                foreach($userPermissions as $permission){
                    if(isset($route['permission']) &&
                        Str::endsWith(strtolower((string) $route['permission']), '*'))
                    {
                        $uriPermission = explode('.',$route['permission']);
                        unset($uriPermission[count($uriPermission) - 1]);

                        if(
                            Str::startsWith($permission,strtolower((string) implode('.',$uriPermission)))
                        ){
                            $filterRoute[] = $route['uri'];
                        }

                    }else{
                        if(isset($route['permission']) && in_array($permission,explode(',',$route['permission']))){
                            $filterRoute[] = $route['uri'];
                        }
                    }
                }
            }
            // 找到角色权限对应的路由
            if(count($filterRoute) > 0 && in_array($routeParams['uri'],$filterRoute)){
                $hasPermission = true;
            }
        });

        return $hasPermission;
    }
}
