<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1\Auth;

use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Services\AuthService;
use App\Services\RoleAndPermissionsService;

class AuthUserController extends Controller
{

    protected $authService = null;

    /**
     * Create a new AuthController instance.
     * @param \App\Services\AuthService $authService
     */
    public function __construct(AuthService $authService)
    {
        $this->authService = $authService;
    }

    /**
     * @param \Illuminate\Http\Request $request
     */
    public function login(Request $request)
    {
        $message = [
            'username.required' => "请输入用户名",
            'password.required' => "请输入密码",
        ];
        $this->validateRequest($request, $message);

        try{
            $response = $this->authService->login($request);
            return Response::success([
                'token' => $response,
            ], '登陆成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * @param \Illuminate\Http\Request $request
     */
    public function logout(Request $request)
    {
        try{
            $this->authService->logout($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }


    public function addUser(Request $request)
    {
        $message = [
            'username.required' => "用户登陆账号必填",
            'password.required' => "密码必填",
            'type.required' => "用户类型必选",
            'nickname.required' => "用户名称必填",
            'email.required' => "邮箱必填",
        ];
        $this->validateRequest($request, $message);
        try{
            $this->authService->saveOrUpdateUser($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function editUser(Request $request)
    {
        $message = [
            'type.required' => "用户类型必选",
            'nickname.required' => "用户名称必填",
            'email.required' => "邮箱必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->authService->saveOrUpdateUser($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }


    public function changeUserStatus(Request $request, $id)
    {
       if(!$id){
           return Response::fail('缺少id参数',500);
       }

       try{
            $this->authService->updateUserStatus($request,$id);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }

    }

    public function delUserByIds(Request $request)
    {
        $message = [
            'ids.required' => "用户id必传",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->authService->deleteUser($request);
            return Response::success(null,'请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 获取用户信息
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function info(Request $request)
    {
        try{
            $user = $this->authService->getUserInfo();
            $service = app(RoleAndPermissionsService::class);
            $service->getRolesToUser();
            return Response::success([
                'userinfo' => $user,
            ], '请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function listByPage(Request $request)
    {
        try{
            $service = app(AuthService::class);
            $roles = $service->listByPage($request);
            return Response::success($service->formatKeysfromArray($roles,'toCamelCase'), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }


    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function resetPassword(Request $request)
    {
        $message = [
            'oldpassword.required' => "请输入旧密码",
            'newpassword.required' => "请输入新密码",
        ];
        $this->validateRequest($request, $message);
        try{
            $response = $this->authService->resetPassword($request);
            return Response::success([], '更新成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }
}
