<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1;

use App\Services\MenusService;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Services\RoleAndPermissionsService;

class PermissionsController extends Controller
{

    protected $roleAndPermissionService = null;

    /**
     * Create a new AuthController instance.
     * @param \App\Services\AuthService $authService
     */
    public function __construct(RoleAndPermissionsService $roleAndPermissionsService)
    {
        $this->roleAndPermissionService = $roleAndPermissionsService;
    }

    /**
     * 获取菜单列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|void
     */
    public function getUserRoleMenuList(Request $request)
    {
        try{
            $menuList = $this->roleAndPermissionService->getUserRoleMenus();
            return Response::success([
                'data' => app(MenusService::class)->getMenusTree($menuList,'page'),
            ], '请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 获取所有菜单列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getAllMenuList(Request $request)
    {
        try{
            $service = app(MenusService::class);
            $menuList = $service->getAllMenus();
            $treeMenu = $service->getMenusTree($menuList);
            return Response::success([
                'data' => $service->formatKeysfromArray($treeMenu,'toCamelCase'),
            ], '请求成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 更新菜单
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function editMenus(Request $request)
    {

        $message = [
            'id.required' => "菜单id必选",
            'name.required' => "菜单名称必填",
            'path.required' => "菜单路径必填",
            'menuType.required' => "菜单类型必选",
            'status.required' => "菜单状态必选",
            'sort.required' => "菜单排序必填",
        ];
        $this->validateRequest($request, $message);

        if($request->get('selected') != true){
            return Response::fail('该菜单未被选中编辑');
        }
        if($request->input('menuType') == 'page'){
            $this->validateRequest($request, [
                'component.required' => "菜单前端组件必填",
                'parentId.required' => "上级菜单必选",
            ]);
        }

        if($request->has('menuType') && 'button' != $request->input('menuType')){
            $message = [
                'icon.required' => "菜单icon必填",
            ];
            $this->validateRequest($request, $message);
        }

        try{
            $service = app(MenusService::class);
            $service->editMenu($request);
            return Response::success([], '更新成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    public function getDefaultButtonPermission(Request $request)
    {
        try{
            $buttonPermissions = $this->roleAndPermissionService->getButtonDefaultPermission();
            return Response::success($buttonPermissions, '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 添加子菜单
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function addSubMenus(Request $request)
    {
        $message = [
            'type.required' => "菜单类型必选",
            'parentId.required' => "上级菜单必填",
            'level.required' => "菜单深度必填",
            'sort.required' => "菜单排序必填",
            'status.required' => "菜单状态必选",
            'path.required' => "菜单路径必填",
            'title.required' => "菜单名称必填",
        ];
        $this->validateRequest($request, $message);

        if('page' == $request->get('type') && $request->get('level') == 2){
            if(empty($request->get('icon'))){
                return Response::fail('菜单图标必选');
            }
            if(empty($request->get('component'))){
                return Response::fail('菜单前端组件必填');
            }
        }

        if('button' == $request->get('type') && $request->get('level') == 3){
            if(empty($request->get('buttonType'))){
                return Response::fail('菜单行为必选');
            }
        }

        try{
            $service = app(MenusService::class);
            $service->addSubMenu($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 删除菜单
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function deleteMenus(Request $request)
    {
        $message = [
            'ids.required' => "菜单id必选",
        ];
        $this->validateRequest($request, $message);

        try{
            $service = app(MenusService::class);
            $service->deleteMenus($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 获取角色列表（页面列表显示）
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getAllByPage(Request $request)
    {
        try{
            $roles = $this->roleAndPermissionService->getAllRolesToPage($request);
            return Response::success($this->roleAndPermissionService->formatKeysfromArray($roles,'toCamelCase'), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getAllList(Request $request)
    {
        try{
            $roles = $this->roleAndPermissionService->getAllRolesList($request);
            return Response::success($this->roleAndPermissionService->formatKeysfromArray($roles,'toCamelCase'), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),$exception->getCode());
        }
    }

    /**
     * 添加角色
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function addRole(Request $request)
    {
        $message = [
            'name.required' => "角色名称必填",
            'remark.required' => "角色描述必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->roleAndPermissionService->saveRole($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 编辑权限
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function editRole(Request $request)
    {
        $message = [
            'id.required' => "角色id必填",
            'name.required' => "角色名称必填",
            'remark.required' => "角色描述必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->roleAndPermissionService->saveRole($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 分配角色权限
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function editRolePermission(Request $request)
    {
        $message = [
            'roleId.required' => "角色id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->roleAndPermissionService->syncPermissionToRole($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function editRoleDefault(Request $request)
    {
        $message = [
            'id.required' => "角色id必填",
            'isDefault.required' => "是否默认必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->roleAndPermissionService->setRoleDefault($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function delByIds(Request $request)
    {
        $message = [
            'ids.required' => "角色id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->roleAndPermissionService->deleteRoleByIds($request);
            return Response::success([], '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

}
