<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Contracts\UsersRepository;
use App\Repositories\Criteria\UserCriteria;
use App\Repositories\Presenters\UserPresenter;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Support\Traits\Helpers;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Spatie\Permission\Models\Role;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

use App\Repositories\Models\AdminUsers;
use Tymon\JWTAuth\Exceptions\TokenInvalidException;

class AuthService
{

    use Helpers;

    protected $userRepository = null;

    public function __construct(UsersRepository $usersRepository)
    {
       $this->userRepository = $usersRepository;
    }

    /**
     * @param Request $request
     */
    public function login(Request $request)
    {
        $username = $request->input('username');
        $password = $request->input('password');

        $requestCriteria = [
            'username' => $username,
            'password' => $password
        ];

        try{
            if( !$user = auth()->attempt($requestCriteria) ){
                throw new UnauthorizedHttpException('authroization error','用户登陆授权失败，用户名或密码错误',null,ResponseCodeEnum::CLIENT_PARAMETER_ERROR);
            }
            if(auth()->user()->toArray()['status'] == 0){
                throw new UnauthorizedHttpException('authroization error','用户登陆授权失败，该用户已被冻结',null,ResponseCodeEnum::CLIENT_PARAMETER_ERROR);
            }
            return $this->generateToken($user);
        }catch(\Exception $exception){
            throw new UnauthorizedHttpException('authroization error',$exception->getMessage(),null,ResponseCodeEnum::SERVICE_LOGIN_ERROR);
        }
    }

    /**
     * 获取登陆用户信息
     * @return \Illuminate\Contracts\Auth\Authenticatable|null
     */
    public function getUserInfo()
    {
        $user = auth()->user();
        //除去敏感信息
        if(isset($user['password'])){
            unset($user['password']);
        }
        if(isset($user['token'])){
            unset($user['token']);
        }

        return $user;
    }


    /**
     * 用户列表
     * @param $request
     * @return array
     */
    public function listByPage($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        if(!$params['page_size'] || $params['page_size'] == 0){
            $params['page_size'] = 10;
        }

        $this->userRepository->pushCriteria(new UserCriteria($request));
        $users = $this->userRepository->paginate($params['page_size'],['id','username','nickname','email','token','is_admin','status','lastlogin','created_at','updated_at']);
        $returnRecord = [];
        if($users){
            $allItems = $users->items();

            $allItems = collect($allItems)->map(function($item){
                $rolesName = [];
                $roles = collect($item->roles->toArray())->map(function($role) use (&$rolesName){
                    $rolesName[] = $role['name'];
                    unset($role['pivot']);
                    return $role;
                });

                $item = $item->toArray();
                unset($item['roles']);
                $item['roles'] = $roles->toArray();
                $item['type'] = $item['is_admin'];
                $item['belongTo'] = implode(',',$rolesName);

                return $item;
            });

            // 获取当前角色的所有权限
            $returnRecord['data'] = $this->formatKeysfromArray($allItems);
            $returnRecord['current_page'] = $users->currentPage();
            $returnRecord['total'] = $users->total();
        }

        return $returnRecord;
    }

    /**
     * 用户登出
     * @param Request $request
     */
    public function logout(Request $request)
    {
        auth()->logout();
    }

    /**
     * 刷新token
     * @return mixed
     * @throws TokenInvalidException
     */
    public function refreshToken()
    {
        $token = auth()->refresh();
        if(!$token){
            throw new TokenInvalidException('token刷新失败',ResponseCodeEnum::SERVICE_TOKEN_REFRESH_ERROR);
        }
        return $this->generateToken($token);
    }

    /**
     * 更新或创建新用户
     * @param $request
     * @return bool
     */
    public function saveOrUpdateUser($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $rule = $this->userRepository->rules['create'];
        $filterRuleParams = [];
        if(isset($params['id']) && !empty($params['id'])){
           $rule = $this->userRepository->rules['update'];
            $filterRuleParams['id'] = $params['id'];
        }

        //整理请求过来的数据参数
        collect($rule)->map(function($r,$key) use ($params, &$filterRuleParams){
            if(isset($params[$key]) && !empty($params[$key])){
                $filterRuleParams[$key] = $params[$key];
            }else{
                $filterRuleParams[$key] = '';
            }
        });
        if($user = $this->userRepository->createOrUpdateUser($filterRuleParams)){
            // 如果编辑用户时选择了所属角色
            if(isset($params['role_ids']) && !empty($params['role_ids'])){
                $ids = explode(',',$params['role_ids']);
                //获取需要分配的角色
                $toBeAssignRoles = Role::query()->whereIn('id',$ids)->get();
                if(!$toBeAssignRoles){
                    throw new \LogicException('角色不存在');
                }

                //判断当前用户是否有相应的角色权限
                //如果有，先取消。然后在进行新的角色权限分配
                $belongRoles = $user->roles;
                //角色
                if(count($belongRoles->toArray()) > 0){
                    $this->removeUserRoleAndPermission($user, $belongRoles);
                }
                $this->authroizeUserRoleAndPermission($user, $toBeAssignRoles);
            }else{
                $belongRoles = $user->roles;
                if(count($belongRoles->toArray()) > 0){
                    $this->removeUserRoleAndPermission($user,$belongRoles);
                }
                // 设置默认角色
                $defaultRole = Role::query()->where('is_default', 1)->get();
                $this->authroizeUserRoleAndPermission($user,$defaultRole);
            }
        }

        return true;
    }

    /**
     * 移除当前用户所在的角色及角色权限
     * @param $user
     * @param $roles
     */
    public function removeUserRoleAndPermission($user,$roles)
    {
        collect($roles)->map(function($role) use ($user){
            $permissions = $role->permissions;
            $user->removeRole($role);
            collect($permissions)->map(function($permission) use ($user){
                $user->revokePermissionTo($permission);
            });
        });
    }

    /**
     * 授权当前用户所在的角色及角色权限
     * @param $user
     * @param $roles
     */
    public function authroizeUserRoleAndPermission($user, $roles)
    {
        collect($roles)->map(function($role) use ($user){
            $user->assignRole($role);
            $permissions = $role->permissions;
            collect($permissions)->map(function($permission) use ($user){
                $user->givePermissionTo($permission);
            });
        });
    }

    /**
     * 更新用户状态
     * @param $request
     * @param $id
     */
    public function updateUserStatus($request, $id)
    {
        $status = 0;
        if(Str::contains(strtolower((string) $request->getPathInfo()), 'disable')){
            $status = 0;
        }elseif(Str::contains(strtolower((string) $request->getPathInfo()), 'enable')){
            $status = 1;
        }

        $this->userRepository->updateUserStatus($id,$status);
    }


    /**
     * 物理删除用户
     * @param $request
     */
    public function deleteUser($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $ids = explode(',',$params['ids']);

        $users = $this->userRepository->whereIn('id',$ids)->get();
        if(count($users->toArray()) != count($ids)){
            throw new \LogicException('部分用户不存在');
        }

        if($this->userRepository->deleteUser($ids)){
            //将用户的角色收回
            collect($users)->map(function($user){
                $roles = $user->roles;
                collect($roles)->map(function($role) use ($user){
                    $user->removeRole($role);
                });
            });
        }
    }


    /**
     * 重置密码
     * @param Request $request
     * @return bool
     */
    public function resetPassword(Request $request)
    {
        // 判断旧密码是否正确
        $oldPassword = $request->input('oldpassword');
        $newPassword = $request->input('newpassword');
        $adminModel = app(AdminUsers::class);
        $user = $this->userRepository->where('id',auth()->user()->id)->first();
        if(!$user){
            throw new UnauthorizedHttpException('user not exist. ','不存在该用户',null,ResponseCodeEnum::SERVICE_USER_NOT_EXIST_ERROR);
        }
        $encodeOldPassword = $adminModel->encryptPassword($oldPassword);
        if($encodeOldPassword != $user->password){
            throw new UnauthorizedHttpException('password is error. ','密码更新失败:旧密码不正确',null,ResponseCodeEnum::SERVICE_PASSWORD_ERROR);
        }
        $user->password = $adminModel->encryptPassword($newPassword);
        if($encodeOldPassword == $user->password){
            throw new UnauthorizedHttpException('password is error. ','新密码不能和旧密码一致',null,ResponseCodeEnum::SERVICE_PASSWORD_ERROR);
        }
        // 根据role规则重组 user
        $updateUser = collect($this->userRepository->rules['update'])->map(function($rule,$key) use ($user){
            return $user[$key];
        });
        $result = $this->userRepository->update($updateUser->toArray(),$user->id);

        return true;
    }
}
