<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Menus;
use App\Support\Traits\Helpers;
use App\Repositories\Contracts\MenusRepository;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

use Tymon\JWTAuth\Exceptions\TokenInvalidException;

class MenusService
{

    use Helpers;

    protected $menusRepository = null;

    public function __construct(MenusRepository $menuRepository)
    {
       $this->menusRepository = $menuRepository;
    }

    /**
     * 添加子菜单
     * @param $request
     * @return bool
     */
    public function addSubMenu($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $menuModel = app(Menus::class);

        $parentMenu = $menuModel->where('id',$params['parent_id'])->first();
        if(!$parentMenu){
            throw new \LogicException('上级菜单不存在！',500);
        }

        if('button' == $params['type']){
            $permission = Permission::query()->where(['menu_id' => 0, 'sys_default' => 1,'guard_name' => config('auth.defaults.guard'), 'action' => $params['button_type']])->first();
            if(!$permission){
                throw new \LogicException('按钮动作不存在！',500);
            }
        }

        $filterParams = [];
        collect($this->menusRepository->rules['create'])->map(function($rule,$key) use ($params, &$filterParams){
            if(isset($params[$key]) && !empty($params[$key])){
                $filterParams[$key] = $params[$key];
            } else{
                $filterParams[$key] = '';
            }
        });
        $filterParams['created_by'] = auth()->user()->id;
        $filterParams['menu_icon'] = $params['icon'];
        $filterParams['component'] = $params['component'];
        $filterParams['status'] = intval($params['status']);

        $filterParams = collect($filterParams)->map(function($value,$key) use ($params){
            if($value == ''){
                switch($key){
                    case 'menu_name' : $value = $params['name']; break;
                    case 'menu_path' : $value = $params['path']; break;
                    case 'menu_icon' : $value = $params['icon']; break;
                    case 'menu_type' : $value = $params['type']; break;
                    case 'is_show' : $value = 1; break;
                    case 'sys_default' : $value = 0; break;
                }
            }
            return $value;
        });

        $result = $this->menusRepository->create($filterParams->toArray());
        if($result){

            $currentUri = $request->route()[1]['uri'];
            if($currentUri){
                $currentUri = str_replace('/','.',$currentUri);
            }

            $permission = [];
            $permission['name'] = $result['menu_name'];
            $permission['menu_id'] = $result['id'];
            if(isset($params['type']) && !empty($params['type']) && 'button' == $params['type']){
                if(Str::endsWith(strtolower((string) $request['name']), $params['button_type'])){
                    $permission['action'] = str_replace('_','.',$request['name']);
                }else{
                    $permission['action'] = str_replace('_','.',$request['name']).'.'.$params['button_type'];
                }
                $permission['permission_type'] = 'button';
            }else{
                $permission['permission_type'] = 'menu';
                $permission['action'] = str_replace('_','.',$request['name']).'.list';
            }
            $permission['sys_default'] = 0;
            $permission['remark'] = $result['menu_name'].'的相关权限';
            $permission['guard_name'] = config('auth.defaults.guard');
            //增加对应的按钮权限
            $permissionResult = Permission::create($permission);
            // 如果是管理员，自动绑定赋予权限
            if(auth()->user()->isSuperAdmin()){
                if($permissionResult){
                    app(RoleAndPermissionsService::class)->assignRoleAndPermissionToUser($permissionResult, auth()->user());
                }
            }
        }
        return true;
    }

    /**
     * 删除菜单
     * @param $request
     */
    public function deleteMenus($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $menuModel = app(Menus::class);
        $menu_ids = explode(',',$params['ids']);
        if(!$menus = $menuModel->whereIn('id',$menu_ids)->get()){
            throw new \LogicException('选中的菜单不存在!',500);
        }
        //判断该菜单下面是否有其他子菜单
        $parent_menus = $menuModel->whereIn('id',$menu_ids)->get();
        if($parent_menus){
            $notFatherMenus = [];
            $checkedMenus = [];
            collect($parent_menus)->map(function($menu) use (&$notFatherMenus, &$checkedMenus){
                if(count($notFatherMenus) == 0){
                    $notFatherMenus[$menu['id']] = $menu;
                }else{
                    if(isset($notFatherMenus[$menu['parent_id']])){
                        unset($notFatherMenus[$menu['parent_id']]);
                        $checkedMenus[] = $menu['parent_id'];
                    }else{
                        if(!in_array($menu['parent_id'],$checkedMenus)){
                            $notFatherMenus[$menu['id']] = $menu;
                        }
                    }
                }
            });
            if(count($notFatherMenus) > 0){
                if(count($menuModel->whereIn('parent_id',array_keys($notFatherMenus))->get()->toArray()) > 0 ){
                    throw new \LogicException('选中的菜单中有子菜单存在，无法删除该菜单。',500);
                }
            }
        }

        //判断选中的菜单是否已被赋予权限给到用户组
        //查询该菜单对应的权限数据
        $menuPermission = Permission::query()->whereIn('menu_id',$menu_ids)->get();
        if($menuPermission){
            $allRoles = Role::all();
            if($allRoles){
                $hasPermission = false;
                collect($allRoles)->map(function($role) use ($menuPermission, &$hasPermission){
                    collect($menuPermission)->map(function($permission) use ($role, &$hasPermission){
                        if($role->hasPermissionTo($permission)){
                            $hasPermission = true ;
                        }
                    });
                });
                if($hasPermission){
                    throw new \LogicException('用户权限中已经包含该菜单权限，请先取消角色中该菜单的权限，在进行删除操作.',500);
                }
                //删除对应的菜单
                collect($menus)->map(function($menu) use ($menuPermission){
                    $menu->delete();
                    //删除对应的菜单权限
                    collect($menuPermission)->map(function($permission){
                        $permission->delete();
                    });
                });
            }
        }
    }

    /**
     * 编辑菜单
     * @param Request $request
     * @return mixed
     */
    public function editMenu(Request $request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $menuModel = app(Menus::class);
        $menu = $menuModel->where('id',$params['id'])->first();
        if(!$menu){
            throw new \LogicException('此菜单不存在！',500);
        }
        $parentMenu = $menuModel->where('id',$params['parent_id'])->first();
        if(!$parentMenu && 'top' != $params['menu_type']){
            throw new \LogicException('上级菜单不存在！',500);
        }
        if(!in_array($params['menu_type'],['top','page','button'])){
            throw new \LogicException('菜单类型不正确！',500);
        }

        $params['menu_name'] = $params['name'];
        $params['menu_path'] = $params['path'];

        // 根据role规则重组 user
        $updateMenus = [];
        collect($this->menusRepository->rules['update'])->map(function($rule,$key) use ($params, &$updateMenus){
            if($key != 'id'){
                $updateMenus[$key] = $params[$key];
            }
        });
        $updateMenus['menu_icon'] = $params['icon'];

        // 处理action
        //app(RoleAndPermissionsService::class)->checkRequestRolesPermissions($request);

        if($params['menu_type'] == 'top'){
            $updateMenus['component'] = '#';
        }else{
            $updateMenus['component'] = $params['component'];
        }
        $updateMenus['created_by'] = auth()->user()->id;
        $result = $this->menusRepository->update($updateMenus,$params['id']);

        if($result){
            $permission = [];
            $permission['name'] = $result['menu_name'];
            $permission['menu_id'] = $result['id'];
            if(isset($params['menu_type']) && !empty($params['menu_type']) && 'button' == $params['menu_type']){
                if(Str::endsWith(strtolower((string) $request['name']), $params['button_type'])){
                    $permission['action'] = str_replace('_','.',$request['name']);
                }else{
                    $permission['action'] = str_replace('_','.',$request['name']).'.'.$params['button_type'];
                }
                $permission['permission_type'] = 'button';
            }else{
                $permission['permission_type'] = 'menu';
                $permission['action'] = str_replace('_','.',$request['name']).'.list';
            }
            $permission['sys_default'] = 0;
            $permission['remark'] = $result['menu_name'].'的相关权限';
            $permission['guard_name'] = config('auth.defaults.guard');
            //增加对应的按钮权限
            if($menuPermission = Permission::query()->where('menu_id',$result['id'])->first()){
                $permissionResult = Permission::query()->updateOrCreate(['id' => $menuPermission->id],$permission);
            }
            // 如果是管理员，自动绑定赋予权限
            if(auth()->user()->isSuperAdmin()){
                if($permissionResult){
                    app(RoleAndPermissionsService::class)->assignRoleAndPermissionToUser($permissionResult, auth()->user());
                }
            }
        }

        return $result;
    }

    /**
     * 获取所有菜单列表
     * @return array
     * @throws \Psr\Container\ContainerExceptionInterface
     *
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function getAllMenus()
    {
        $menuModel = app(Menus::class);
        $allMenus = $menuModel->get();
        // key_name转换
        $parent_id = [];
        $menusList = collect($allMenus)->map(function($menu) use (&$parent_id){

            $newMenu = [
                'name' => $menu->menu_name,
                'title' => $menu->title,
                'path' => $menu->menu_path,
                'icon' => $menu->menu_icon,
                'expand' => true,
                'checked' => false
            ];

            if($menu->parent_id > 0 && $menu->menu_type != 'top'){
                $parent_id[] = $menu->parent_id;
            }

            $menuArr = $menu->toArray();

            unset($menuArr['menu_name']);
            unset($menuArr['menu_path']);
            unset($menuArr['menu_icon']);

            return array_merge($newMenu,$menuArr);
        });

        $parentMenu = $menuModel->whereIn('id',$parent_id)->get();
        $menusList = collect($menusList)->map(function($menu) use ($parentMenu){
            $parent_id = $menu['parent_id'];
            if($parent_id > 0){
                $parentName = collect($parentMenu)->map(function($m) use (&$menu,$parent_id){
                    if($m->id == $parent_id){
                        $menu['parent_title'] = $m->title;
                    }
                });
            }
            return $menu;
        });

        return $menusList->toArray();
    }

    /**
     * 获取菜单结构树
     * level : all (全部) top (只显示顶部) page (只显示2级,不包括button级)
     * @param $menus
     * @return false|mixed
     */
    public function getMenusTree($menus, $level = 'all')
    {
        $treeMenus = [];
        collect($menus)->map(function($menu) use (&$treeMenus, $level){
            if($menu['parent_id'] == -1 && 'top' == $menu['menu_type']){
                $treeMenus[$menu['id']] = $menu;
                $treeMenus[$menu['id']]['level'] = 0;
            }else{
                if('top' == $level){
                    return ;
                }elseif('page' == $level){
                    if('page' == $menu['menu_type']) {
                        if($menu['parent_id'] == 0) {
                            $treeMenus[$menu['id']] = $menu;
                            $treeMenus[$menu['id']]['level'] = 1;
                        }else{
                            $treeMenus = $this->getChildrenTree($treeMenus,$menu);
                        }
                    }
                }else{
                    if($menu['parent_id'] == 0) {
                        $treeMenus[$menu['id']] = $menu;
                        $treeMenus[$menu['id']]['level'] = 1;
                    }else{
                        $treeMenus = $this->getChildrenTree($treeMenus,$menu);
                    }
                }
            }
        });

        return array_values($treeMenus);
    }

    /**
     * 递归树形结构菜单
     * @param $treeMenus
     * @param $currMenu
     * @return array
     */
    private function getChildrenTree($treeMenus,$currMenu) : array
    {
        $returnTreeMenus = collect($treeMenus)->map(function($tMenus) use ($currMenu){
            if(!isset($tMenus['children']) || empty($tMenus['children'])){
                if($currMenu['parent_id'] == $tMenus['id']){
                    $currMenu['level'] = (int)$tMenus['level'] + 1;
                    $tMenus['children'][] = $currMenu;
                }
            } // 如果有children的话，进行递归循环
            else{
                if($currMenu['parent_id'] == $tMenus['id']){
                    $currMenu['level'] = (int)$tMenus['level'] + 1;
                    $tMenus['children'][] = $currMenu;
                }else{
                    $tMenus['children'] = $this->getChildrenTree($tMenus['children'],$currMenu);
                }
            }

            return $tMenus;
        });

        return $returnTreeMenus->toArray();
    }
}
