<?php /*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Repositories\Models\Menus;
use App\Support\Traits\Helpers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Spatie\Permission\Exceptions\RoleDoesNotExist;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;
use App\Repositories\Contracts\MenusRepository;
use App\Repositories\Contracts\RolesRepository;


class RoleAndPermissionsService
{
    use Helpers;

    protected $user = null;
    protected $menusRepository = null;

    public function __construct(MenusRepository $menusRepository)
    {
        $this->menusRepository = $menusRepository;
    }

    /**
     * 获取管理后台用户的所在的用户组
     */
    public function getRolesToUser($user = null){
        if($user == null){
            $user = auth()->user();
        }
        $this->getAllPermissions($user);
        $role = $user->roles;

        return $role;
    }

    public function getUserPermissions(){
        $role = $this->getAllPermissions(auth()->user());
        return $role;
    }

    /**
     * 获取用户所有权限
     * @param $user
     * @param bool $parseUserRole
     * @return mixed
     */
    protected function getAllPermissions($user, $parseUserRole = true)
    {
        $role = $user->getpermissionsviaroles();
        if($parseUserRole){
            $newRoles = collect($user['roles'])->map(function($role) {
                $roleArray = $role->toArray();
                unset($roleArray['pivot']);
                unset($roleArray['guard_name']);
                $newPermission = collect($roleArray['permissions'])->map(function ($permission)  {
                    // 隐藏系统默认权限
                    if ($permission['menu_id'] > 0) {
                        unset($permission['pivot']);
                        unset($permission['guard_name']);
                        //$newPermission[] = $permission;
                        return $permission;
                    }
                });
                $roleArray['permissions'] = array_values(array_filter($newPermission->toArray()));

                return $roleArray;
            });
            unset($user['roles']);
            $user['roles'] = $newRoles->toArray();
            //将权限赋予当前用户
            auth()->setUser($user);
        }

        return $role;
    }


    /**
     * 获取系统默认级别的按钮权限类型
     * @return array|\Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection
     */
    public function getButtonDefaultPermission()
    {
        $allPermission = Permission::query()->where(['menu_id' => 0, 'sys_default' => 1,'guard_name' => config('auth.defaults.guard')])->get();
        if($allPermission){
            collect($allPermission)->map(function($permission){
                $permission->title = $permission->name;
            });
            $allPermission = $this->formatKeysfromArray($allPermission);
        }

        return $allPermission;
    }

    /**
     * 获取用户授权菜单列表
     * @return array
     */
    public function getUserRoleMenus()
    {
        $permissions = $this->getAllPermissions(auth()->user());
        $menuIds = collect($permissions)->map(function($permission){
            if($permission instanceof \Spatie\Permission\Models\Permission){
                $permission = $permission->toArray();
            }
            return $permission['menu_id'];
        });

        $menuModel = app(Menus::class);
        $menuList = $menuModel
            ->whereIn($menuModel->getKeyName(),$menuIds)
            ->where('status',1)->get()->toArray();

        $menuList = collect($menuList)->map(function($menu){
            $menu['showAlways'] = true;
            return $menu;
        });

        return empty($menuList) || NULL ? [] : $menuList;
    }

    /**
     * 同步指定权限给指定用户
     * @param $permission
     * @param $user
     */
    public function assignRoleAndPermissionToUser($permission, $user)
    {
        $roles = $this->getRolesToUser($user);
        if($roles == null){
            throw new \LogicException('该用户没有所属用户组,请手动进行分组在分配权限!');
        }
        $roleArr = collect($roles)->map(function($role){
            return $role['id'];
        });
        // 获取当前用户的组信息
        $allRoles = Role::query()->whereIn( 'id' , $roleArr->toArray())->get();
        if($allRoles){
            collect($allRoles)->map(function($role) use ($permission,$user){
                $role->givePermissionTo($permission);
                $user->givePermissionTo($permission);
            });
        }
    }

    /**
     * 获取所有角色并获取对应的所有权限(用于列表展示)
     * @param $request
     * @return array
     */
    public function getAllRolesToPage($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        if(!$params['page_size'] || $params['page_size'] == 0){
           $params['page_size'] = 10;
        }
        $roles = app(RolesRepository::class)->paginate($params['page_size'],['id','name','is_default','remark','created_at','updated_at']);
        if($roles){
            $rolePermission = [];
            $items = collect($roles->items())->map(function($role) use (&$rolePermission){
                $permissions = $role->permissions->toArray();
                $permissions = collect($permissions)->map(function($permission){
                    if($permission['sys_default'] == 1 && $permission['menu_id'] == 0){
                        unset($permission);
                    }else{
                        unset($permission['pivot']);
                        $permission['created_at'] = date('Y-m-d H:i:s',strtotime($permission['created_at']));
                        $permission['updated_at'] = date('Y-m-d H:i:s',strtotime($permission['updated_at']));
                        return $permission;
                    }
                });
                $rolePermission[$role->id] = array_values(array_filter($permissions->toArray()));

                return $role;
            });

            $allItems = collect($items)->map(function($item) use ($rolePermission){
                $itemArr = $item->toArray();
                $itemArr['is_default'] = $itemArr['is_default'] == 1 ? true : false;
                $itemArr['created_at'] = date('Y-m-d H:i:s',strtotime($itemArr['created_at']));
                $itemArr['updated_at'] = date('Y-m-d H:i:s',strtotime($itemArr['updated_at']));
                unset($itemArr['permissions']);
                $itemArr['permissions'] = $rolePermission[$item['id']];

                return $itemArr;
            });

            // 获取当前角色的所有权限
            $returnRecord['data'] = $this->formatKeysfromArray($allItems);
            $returnRecord['current_page'] = $roles->currentPage();
            $returnRecord['total'] = $roles->total();
        }
        return $returnRecord;
    }

    /**
     * 仅获取所有角色列表
     * @param $request
     * @return array
     */
    public function getAllRolesList($request)
    {
        $roles = Role::all();
        return $roles->toArray();
    }

    /**
     * 保存角色
     * @param $request
     * @return bool
     */
    public function saveRole($request){
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        try{
            $role = Role::query()->where('name', $params['name'])->first();
            if(!empty($role)) {
                $role = $role->toArray();
            }
            if(!isset($params['id']) || empty($params['id'])){
                if(!empty($role)){
                    throw new \LogicException('角色名称不能重复',500);
                }
            }else{
                if(!empty($role) && $role['id'] != $params['id']){
                    throw new \LogicException('角色名称不能重复',500);
                }
            }
        }catch(\Exception $exception){
            if (!$exception instanceof RoleDoesNotExist){
                throw $exception;
            }
        }
        app(RolesRepository::class)->saveRole($params);

        return true;
    }

    /**
     * 分配权限给相应角色
     * @param $request
     * @return bool
     */
    public function syncPermissionToRole($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        //判断该角色是否存在
        $role = Role::findById($params['role_id']);
        if(!$role){
            throw new \LogicException('角色不存在');
        }
        if(isset($params['perm_ids']) && !empty($params['perm_ids'])){
            $permisson_ids = explode(',',$params['perm_ids']);
            $permission = Permission::query()->whereIn('menu_id',$permisson_ids)
                ->where(['guard_name' => config('auth.defaults.guard'),'sys_default' => 0])->get();
            if(count($permission) != count($permisson_ids)){
                throw new \LogicException('部分权限不存在');
            }
            if(count($role->permissions->toArray()) > 0){
                //先作废原先的权限
                $role->revokePermissionTo($role->permissions);
            }
            $role->givePermissionTo($permission);
        }else{
            $role->revokePermissionTo($role->permissions);
        }

        return true;
    }

    /**
     * 删除角色
     * @param $request
     */
    public function deleteRoleByIds($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        //判断该角色是否存在
        $roles = Role::query()->whereIn('id',explode(',',$params['ids']))->get();
        if(!$roles){
            throw new \LogicException('角色不存在');
        }
        collect($roles)->map(function($role){
            $role->delete();
            if(count($role->permissions->toArray()) > 0){
                $role->revokePermissionTo($role->permissions);
            }
        });
    }

    /**
     * 设置默认角色组
     * @param $request
     */
    public function setRoleDefault($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        //判断该角色是否存在
        $roles = Role::query()->whereIn('id',explode(',',$params['id']))->first();
        if(!$roles){
            throw new \LogicException('角色不存在');
        }
        $default = 0;
        if($params['is_default'] == true) {
           $default = 1;
        }

        try{
            DB::beginTransaction();
            $defaultRole = Role::query()->where('is_default',1)->first();
            if($defaultRole){
                $defaultRole->update([
                    'is_default' => 0
                ]);
            }
            $roles->update([
                'id' => $params['id'],
                'is_default' => $default
            ]);
            DB::commit();
        }
        catch (\Exception $e) {
            DB::rollBack();
        }
    }
}
