<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class CreatePermissionTables extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        $tableNames = config('permission.table_names');
        $columnNames = config('permission.column_names');

        if (empty($tableNames)) {
            throw new \Exception('Error: config/permission.php not loaded. Run [php artisan config:clear] and try again.');
        }

        /*
         * 权限表
         * 所有的菜单、搜索、列表中的操作权限都在这里。
         */
        Schema::create($tableNames['permissions'], function (Blueprint $table) {
            $table->id();
            $table->string('name');
            $table->unsignedBigInteger('menu_id');
            $table->string('action')->comment('权限的行为');
            $table->enum('permission_type',['menu','data','button'])->nullable()->comment('权限类型');
            $table->unsignedTinyInteger('sys_default')->default(0)->comment('是否系统默认');
            $table->text('remark')->nullable()->comment('备注');
            $table->string('guard_name')->comment('权限插件保留字段');

            $table->timestamps();
        });
        $columns = ['name','guard_name'];
        Schema::table($tableNames['permissions'],function(Blueprint $table) use ($tableNames,$columns){
            DB::statement('ALTER TABLE '.$tableNames['permissions']. ' ROW_FORMAT=DYNAMIC;');
            //外键关联menus表id
            /*$table->foreign('menu_id')
                ->references('id')
                ->on('menus')
                ->onUpdate('CASCADE')
                ->onDelete('CASCADE');*/
            $table->index($columns);
            $table->unique($columns);
        });

        Schema::create($tableNames['roles'], function (Blueprint $table) {
            $table->id();
            $table->string('name')->comment('名称');
            $table->unsignedTinyInteger('is_default')->default(0)->comment('是否默认');
            $table->text('remark')->nullable()->comment('备注');
            $table->string('guard_name')->comment('权限插件保留字段');

            $table->timestamps();
        });
        $columns = ['name','guard_name'];
        Schema::table($tableNames['roles'],function(Blueprint $table) use ($tableNames,$columns){
            DB::statement('ALTER TABLE '.$tableNames['roles']. ' ROW_FORMAT=DYNAMIC;');
            $table->index($columns);
            $table->unique($columns);
        });

        Schema::create($tableNames['model_has_permissions'], function (Blueprint $table) use ($tableNames, $columnNames) {
            $table->unsignedBigInteger('permission_id');

            $table->string('model_type');
            $table->unsignedBigInteger($columnNames['model_morph_key']);

            $table->foreign('permission_id')
                ->references('id')
                ->on($tableNames['permissions'])
                ->onDelete('cascade');
        });
        Schema::table($tableNames['model_has_permissions'],function(Blueprint $table) use ($tableNames,$columnNames){
            DB::statement('ALTER TABLE '.$tableNames['model_has_permissions']. ' ROW_FORMAT=DYNAMIC;');
            $table->index([$columnNames['model_morph_key'], 'model_type'], 'model_has_permissions_model_id_model_type_index');
            $table->primary(['permission_id', $columnNames['model_morph_key'], 'model_type'],
                'model_has_permissions_permission_model_type_primary');
        });

        Schema::create($tableNames['model_has_roles'], function (Blueprint $table) use ($tableNames, $columnNames) {
            $table->unsignedBigInteger('role_id');

            $table->string('model_type');
            $table->unsignedBigInteger($columnNames['model_morph_key']);

            $table->foreign('role_id')
                ->references('id')
                ->on($tableNames['roles'])
                ->onDelete('cascade');

        });
        Schema::table($tableNames['model_has_roles'],function(Blueprint $table) use ($tableNames,$columnNames){
            DB::statement('ALTER TABLE '.$tableNames['model_has_roles']. ' ROW_FORMAT=DYNAMIC;');
            $table->index([$columnNames['model_morph_key'], 'model_type'], 'model_has_roles_model_id_model_type_index');
            $table->primary(['role_id', $columnNames['model_morph_key'], 'model_type'],
                'model_has_roles_role_model_type_primary');
        });


        Schema::create($tableNames['role_has_permissions'], function (Blueprint $table) use ($tableNames) {
            $table->unsignedBigInteger('permission_id');
            $table->unsignedBigInteger('role_id');

            $table->foreign('permission_id')
                ->references('id')
                ->on($tableNames['permissions'])
                ->onDelete('cascade');

            $table->foreign('role_id')
                ->references('id')
                ->on($tableNames['roles'])
                ->onDelete('cascade');

        });
        Schema::table($tableNames['role_has_permissions'],function(Blueprint $table) use ($tableNames,$columnNames){
            DB::statement('ALTER TABLE '.$tableNames['role_has_permissions']. ' ROW_FORMAT=DYNAMIC;');
            $table->primary(['permission_id', 'role_id'], 'role_has_permissions_permission_id_role_id_primary');
;
        });


        app('cache')
            ->store(config('permission.cache.store') != 'default' ? config('permission.cache.store') : null)
            ->forget(config('permission.cache.key'));
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        $tableNames = config('permission.table_names');

        if (empty($tableNames)) {
            throw new \Exception('Error: config/permission.php not found and defaults could not be merged. Please publish the package configuration before proceeding, or drop the tables manually.');
        }

        Schema::drop($tableNames['role_has_permissions']);
        Schema::drop($tableNames['model_has_roles']);
        Schema::drop($tableNames['model_has_permissions']);
        Schema::drop($tableNames['roles']);
        Schema::drop($tableNames['permissions']);
    }
}
